package org.jabber.applet;
import java.awt.*;
import java.util.Vector;
import java.util.Enumeration;
/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * This class is used to group JIDs (or JIDs)
 * 
 * @author David Scott
 */
public class JIDGroup {
    /**
     * Name of the group
     */
    private String name;

    /**
     * Flag indicating state of the view (expanded or shrunk)
     */
    private boolean expanded = true;

    /**
     * Number of members in this group
     */
    private int memberCount = 0;

    /**
     * Vector of JIDs in this group
     */
    private Vector vJIDs = new Vector();

    /**
     * Group constructor
     * Set the name and default all other vars
     * @param name Name of the group
     */
    public JIDGroup(String name) {
        this.name = name;
    }

    /**
     * Get a copy of vector of JIDs in this group
     * @return Vector of JIDs.
     */
    public Vector getJIDs() {
        return ((Vector)vJIDs.clone());
    }

    /**
     * Add a user to this group
     * @param newMember JID describing the member to be added
     */
    public void jidAdd(JID newMember) {
        int ndxSearch = 0;
        int ndxInsert = 0;
        boolean bSlippedIn = false;

        // loop thru the vector
        // as soon as find where new name > vector name, insert
        Enumeration e = vJIDs.elements();
        while (e.hasMoreElements()) {
            Object obj = e.nextElement();
            String sVector = ((JID)obj).getNick();
            if ( newMember.getNick().toUpperCase().compareTo(sVector.toUpperCase()) < 0 ) {
                if (bSlippedIn == false) {
                    ndxInsert = ndxSearch;
                    bSlippedIn = true;
                }
            }
            ndxSearch++;
        }
        if (bSlippedIn == false)
            ndxInsert = ndxSearch;
        vJIDs.insertElementAt(newMember, ndxInsert);
        memberCount++;
    }

    /**
     * Find the specified member in the current group
     * @param sMember Username of member to find
     * @param sServer Server associated with member to be found
     * @return the JID searched for
     */
    public JID getJID(JID jid, int nMask) {
        Enumeration e = vJIDs.elements();
        while (e.hasMoreElements()) {
            Object obj = e.nextElement();
            if (((JID)obj).isEqual(jid, nMask) == true)
                return((JID)obj);
        }
        return null;
    }

    /**
     * Find the nth member in the group
     * @param jindex index into the group of the JID you're interested
     * @return the JID searched for
     */
    public JID getJID(int jindex) {
        int i = -1;
        Enumeration e = vJIDs.elements();
        while (e.hasMoreElements()) {
            i++;
            Object obj = e.nextElement();
            if (jindex == i)
                return((JID)obj);
        }
        return null;
    }

    /**
     * Find index of user
     * @param sMember Username of member to be searched for
     * @param sServer Server corresponding to user to be searched for
     * @return Index into vector corresponding to user searched for
     */
    public int getJIDIndex(JID jid, int nMask) {
        int i = -1;
        Enumeration e = vJIDs.elements();
        while (e.hasMoreElements()) {
            i++;
            Object obj = e.nextElement();
            if (((JID)obj).isEqual(jid, nMask) == true)
                return(i);
        }
        return -1;
    }

    /**
     * Remove a user from the group
     * @param sMember Users JID name
     * @param sServer Users JID server
     * @return true - remove successful
     * false - remove failed
     */
    public boolean jidRemove(JID jid, int nMask) {
        String sMember = jid.getUsername();
        String sServer = jid.getServer();
        int i = getJIDIndex(jid, nMask);
        if (i > -1) {
            vJIDs.removeElementAt(i);
            memberCount--;
            return( true );
        } else
            return( false );
    }

    /**
     * Remove all users from a group
     */
    public void removeAll() {
        vJIDs.removeAllElements();
        memberCount = 0;
    }

    /**
     * Get name of group
     * @return Group name
     */
    public String getName() {
        return (name);
    }

    /**
     * Get number of members in the group with non-empty status.
     * @return Count of members in the group with non-empty status.
     */
    public int getStatusCount() {
        Vector jids = getJIDs();
        int count = 0;
        for( int i= 0; i < jids.size(); i++ ) {
            JID jid = (JID)jids.elementAt(i);
            if( jid.isActive() )
                count++;
        }
        return count;
    }

    /**
     * Get number of members in the group
     * @return Count of members in the group
     */
    public int getJIDCount() {
        return (memberCount);
    }

    /**
     * Get 'expanded' state of group
     * @return true = expanded, false = compressed
     */
    public boolean getExpanded() {
        return expanded;
    }

    /**
     * Set 'expanded' state of group
     */
    public void setExpanded() {
        expanded = true;
    }

    /**
     * Toggle state of expanded flag
     */
    public void toggleExpanded() {
        expanded = !expanded;
    }

    /**
     * Convert group header to string (with count of users in group)
     * @return string consisting of group name and count of users in that group
     */
    public String toString() {
        String exp = " - ";
        String tail = "";
        if( !getExpanded() ) {
            exp = " + ";
            tail = " ( " + getStatusCount() + " of " + getJIDCount() + " )";
        }
        return  exp + getName() + tail;
    }
}
