package org.jabber.applet.awt;

import java.awt.*;
import java.awt.event.*;            // needed for focus manipulation
import java.util.Calendar;          // used to set time of chat start
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import org.jabber.applet.*;
import org.jabber.applet.tools.*;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * Jabber Chat Window class.
 * All chats initiated from the Jabber Applet launch this dialog window.
 * The UI consists of 
 * a) a label at the top of the window indicating person we're chatting with
 * b) a text area displaying history of the chat
 * c) a text field for user input to the chat session
 * d) a send button
 * e) an invite button
 * f) a record button
 * 
 * @author David Scott
 */
public class MsgChat extends Frame implements ActionListener, FocusListener, KeyListener, WindowListener {
    public JabberApplet app=null;

    /**
     * top panel used to display 'chatn with..'
     */
    private MyPanel pnlHeader = new MyPanel();
    private Label lblUsername = new Label("");

    /**
     * chat history display area
     */
    private ChatTextArea txtChatHistory;

    /**
     * new msg input area
     */
    private TextField txtSendMsg;

    /**
     * current state of window is iconified
     */
    private boolean bIconified = false;

    /**
     * Number of messages received while chat window has been iconified
     */
    private int nMsgCount = 0;

    /**
     * String containing my username
     */
    private JID JIDMe;

    /**
     * User we're chatting withs JID
     */
    public JID JIDYou;

    /**
     * if True we will save the chat history to be displayed later
     */
    public boolean mbRecord = false;

    /**
     * Max capacity of String buffer for saving chat history.
     */
    protected static int SB_MAX_CAPACITY = 10000;

    /**
     * Newline for this platform.
     */
    protected static String NEWLINE = System.getProperty("line.separator");

    /**
     * String buffer for saving chat history.
     */
    private StringBuffer historyBuff = new StringBuffer( SB_MAX_CAPACITY );

    /**
     * Used to covert HTML to standard ASCII (for ' etc.)
     */
    public StringCleanup moCleanup = new StringCleanup();

    /**
     * Background color of clients applet
     */
    public Color clientColor;

    /**
     * Font of clients applet
     */
    public Font clientFont;

    /**
     * Background color of clients applet
     */
    private Hashtable mhashNameColors = new Hashtable();

    /**
     * Label of the button with focus
     */
    private String				msButtonWithFocus		= "";

    /**
     * This is true when you start typing a reply and the other person is notified that you started typing
     */
    //private boolean				mbNotifiedStartingReply = false;

    private boolean             bGroupChat = false;

    //Colors choosen from http://www.geocities.com/~annabella/colornames.html
    private UniqueColors clr = new UniqueColors();
    private final static String         COLOR_MSG            = "000000";
    private final static String         COLOR_ACTION         = "8B008B";
    private final static String         COLOR_STATUS         = "2E8B57";
    private Vector						mvChatHistory		 = new Vector(200, 50);
    /**
    * Assign an unique color for everyone in the Chat room.
    */
    private String nameColor(String name) {
        String sColor = (String) mhashNameColors.get(name);
        if (sColor == null) {
            //name not found in the hash table, so add it
            sColor = clr.getNextRGB();
            mhashNameColors.put(name, new String(sColor));
        }
        return sColor;
    }

    /**
    * The MyPanel class is used so we can put some graphics elements
    * in a panel.
    */
    class MyPanel extends Panel {
        /**
         * paint method is used to display the top panel with some
         * basic graphics.  A label is displayed (with users name),
         * that's created in the window constructor.
         * @param g graphics handle for the object to be written on
         */
        public void paint(Graphics g) {
            g.setColor(clientColor);        // same as chats background
            // draw rect around top panel
            g.draw3DRect(2,4,this.getBounds().width-4,this.getBounds().height-8, false);
            // compute size of 'Chat with'
            FontMetrics fnt = g.getFontMetrics();
            int nWidth = fnt.stringWidth("Chat With");
            int nHeight = fnt.getDescent();
            g.setColor(clientColor);        // erase part of outline
            g.fillRect(5, 0, nWidth + 5, 18);   // display background for 'Chat with'

            g.setColor(Color.black);        // display 'Chat with'
            g.drawString("Chat With", 8, 10);
        }                                   // end, paint routine
    }                                       // end, MyPanel class

    Button btnRecord = null;

    /**
     * Main chat constructor (doens't include an invite button)
     * 
     * @param itsMe owners name
     * @param jidYou person chatting with itsMe/owner
     * @param clientColor Client window color
     * @param clientFont Client window font
     */
    public MsgChat(Color clientColor, Font clientFont, String itsMe, JID jidYou, boolean bHeader) {
        this (clientColor, clientFont, itsMe, jidYou, bHeader, false);
    }

    /**
     * Main chat constructor (specify whether you want an invite button or not)
     * @param itsMe owners name
     * @param jidYou person chatting with itsMe/owner
     * @param clientColor Client window color
     * @param clientFont Client window font
     * @param inviteButton if true, display the invite button
     */
    public MsgChat(Color clientColor, Font clientFont, String itsMe, JID jidYou, boolean bHeader, boolean inviteButton) {
        super();
        // get instance of Applet so we can call methods in this class from classes
        // defined in this file.
        app = (JabberApplet) JabberApplet.getAppletInstance();
        JIDMe = new JID(itsMe);           // keep copy of my username Changed from (itsMe, jidYou.getServer())
        JIDYou = jidYou;                 // keep copy of person we're chatting with
        this.clientColor = clientColor;
        this.setBackground(clientColor);
        this.clientFont = clientFont;
        this.setFont(clientFont);
        setBackground(new java.awt.Color(255,255,255));

        if (bHeader) {
            // Header information
            lblUsername.setAlignment(Label.CENTER);
            lblUsername.setText("          " + JIDYou.getNick() + "          "); // set chat to name in header
            // pad with space so that you can see " is replying..."
            pnlHeader.add(lblUsername);             // reserve some space
            pnlHeader.setBackground(clientColor);
        }

        // Chat history
        txtChatHistory = new ChatTextArea();
        txtChatHistory.setFont(clientFont);
        String sTime = "Time: " + Calendar.getInstance().getTime();
        txtChatHistory.append(sTime, COLOR_STATUS);
        txtChatHistory.addFocusListener(this); // workaround to pass focus to the send field
        chatHistory(sTime, COLOR_STATUS);

        // Send Msg field and button
        txtSendMsg = new TextField();
        txtSendMsg.addActionListener(this);    // send and add to history whenever <enter>
        txtSendMsg.addKeyListener(this);

        //Button btnSend = new Button(Config.SEND);   // send button
        Button btnSend = new Button("Send");   // send button
        btnSend.addActionListener(this);
        //btnSend.addKeyListener(this);
        btnSend.addFocusListener(this);


        Button btnInvite = new Button("Invite");   // invite button
        btnInvite.addActionListener(this);
        btnInvite.addFocusListener(this);

        btnRecord = new Button(" Record ");   // Record chat history button
        btnRecord.addActionListener(this);
        btnRecord.addFocusListener(this);

        //----------------------------------
        // Screen layout setup
        //----------------------------------
        int x = 0;
        int y = 0;
        setLayout(new GridBagLayout());

        //----------------------------------
        // add header
        //----------------------------------
        GridBagConstraints gbConstraints=new GridBagConstraints();
        gbConstraints.fill=GridBagConstraints.HORIZONTAL;
        gbConstraints.gridx = x;         // x location
        gbConstraints.gridy = y;         // y location

        bGroupChat = inviteButton;
        if (inviteButton)
            gbConstraints.gridwidth = 4;     // width
        else
            gbConstraints.gridwidth = 3;     // width

        gbConstraints.gridheight = 1;    // height
        gbConstraints.ipadx = 0;         // no xtra padding
        gbConstraints.ipady = 0;         // no xtra padding
        gbConstraints.weightx = 0.0;     // gets no extra x when avail
        gbConstraints.weighty = 0.0;     // gets no extra y when avail

        if (bHeader)
            add
                (pnlHeader,gbConstraints);

        //----------------------------------
        // add text history
        //----------------------------------
        y++;
        gbConstraints.gridx = x;         // x location
        gbConstraints.gridy = y;         // y location
        gbConstraints.weightx = 1.0;     // gets xtra x when avail
        gbConstraints.weighty = 1.0;     // gets xtra y when avail
        gbConstraints.fill=GridBagConstraints.BOTH;
        add
            (txtChatHistory,gbConstraints);

        gbConstraints.weightx = 0.0;     // gets no extra x when avail
        gbConstraints.weighty = 0.0;     // gets no extra y when avail

        //----------------------------------
        // add send field and button
        //----------------------------------
        y++;
        gbConstraints.gridx = x;         // x location
        gbConstraints.gridy = y;         // y location
        gbConstraints.gridwidth = 1;     // width
        gbConstraints.fill=GridBagConstraints.HORIZONTAL;
        gbConstraints.weightx = 1.0;     // gets no extra x when avail
        add
            (txtSendMsg,gbConstraints);

        x++;
        gbConstraints.gridx = x;         // x location
        gbConstraints.weightx = 0.0;     // gets no extra x when avail
        gbConstraints.fill=GridBagConstraints.NONE;
        add
            (btnSend, gbConstraints);


        if (inviteButton) {
            x++;
            gbConstraints.gridx = x;         // x location
            gbConstraints.weightx = 0.0;     // gets no extra x when avail
            gbConstraints.fill=GridBagConstraints.NONE;
            add
                (btnInvite, gbConstraints);
        }

        x++;
        gbConstraints.gridx = x;         // x location
        gbConstraints.weightx = 0.0;     // gets no extra x when avail
        gbConstraints.fill=GridBagConstraints.NONE;
        add
            (btnRecord, gbConstraints);

        //SymWindow aSymWindow = new SymWindow();
        addWindowListener(this);
        SymComponent aSymComponent = new SymComponent();
        this.addComponentListener(aSymComponent);

        // need to pack in the constructor for linux.
        pack();
    }

    //---------- KeyListener Implementation ------------------

    /**
     * keyTyped
     */
public void keyTyped(KeyEvent evt) { }

    /**
     * keyPressed
     */
    public void keyPressed(KeyEvent evt) {
        //[Enter] hit, see if a button has focus, if it does
        //click it, otherwise click the default Send button
        if( evt.getKeyCode() != KeyEvent.VK_DELETE
                && evt.getKeyCode() != KeyEvent.VK_ALT
                && evt.getKeyCode() != KeyEvent.VK_CONTROL
                && evt.isAltDown() && evt.isControlDown() ) {
            JabberApplet.getAppletInstance().toggleDebug();
        }

        if (evt.getKeyCode() == KeyEvent.VK_ENTER ) {
            if (msButtonWithFocus.length() > 0) {
                if (msButtonWithFocus.equals("Send")) {
                    msgSend();
                } else if (msButtonWithFocus.equals("Invite")) {
                    invite();
                } else { //Button caption can be either Record or Stop Rec
                    record(msButtonWithFocus);
                }
            } else {
                msgSend();
            }
        }
        /*
        else 
        {	//TODO verify that this isn't being called when non alpha-numeric keys are pressed
        	if (! mbNotifiedStartingReply) 
        	{
        	    if (bGroupChat == false)  // don't send when in groupchat
        		    app.replyStarted(JIDYou);
        		mbNotifiedStartingReply = true;
        	}
        }
        */
    }

    /**
     * keyReleased
     */
    public void keyReleased(KeyEvent evt) { }

    /**
     * Button triggered.
     * @param ae Action event (not used)
     */
    public void actionPerformed(ActionEvent ae) {
        if (ae.getActionCommand().equals("Send")) {
            msgSend();
        } else if(ae.getActionCommand().equals("Invite")) {
            invite();
        } else { //Button caption can be either Record or Stop Rec
            record(ae.getActionCommand());
        }
    }

    /**
     * Display a textArea for the user to cut/paste from.
     */
    public void displayChatHistory(  ) {
        TextArea ta = new TextArea(chatHistoryToString());
        final Dialog dialog =
            new Dialog(this,"CHAT HISTORY: please cut and paste the message to a text document.", true);
        dialog.add(ta);
        dialog.addWindowListener( new WindowAdapter() {
                                      public void windowClosing( WindowEvent we ) {
                                          dialog.dispose();
                                      }
                                  }
                                );
        dialog.pack();
        dialog.show();
    }

    /**
     * New groupchat msgReceived needs jid input since multiple users are shown in the same 
     * chat window.
     */
    public void msgReceived(JID jid, String sResponse) {
        //---------------------------------------------------
        // If chat windows iconified update the msg counter
        // and update icon displayed
        //---------------------------------------------------
        if (bIconified)                 // window iconified?
            nMsgCount++;                // update msg count
        else
            nMsgCount = 0;              // window showing, clear msg count
        updateCount(nMsgCount);

        if (jid.getNick().length() == 0)  //groupchat
        {
            txtChatHistory.append(sResponse, COLOR_STATUS);
            chatHistory(sResponse, COLOR_STATUS);
        } else {
            if (sResponse.startsWith("/me")) {	//Action Message
                //Remove "/me"
                sResponse = moCleanup.Cleanup(sResponse.substring(3));
                txtChatHistory.append("* " + jid.getNick() + " " + moCleanup.Cleanup(sResponse), COLOR_ACTION);
                chatHistory(jid.getNick() + " " + moCleanup.Cleanup(sResponse), COLOR_ACTION);
            } else {
                sResponse = moCleanup.Cleanup(sResponse);
                txtChatHistory.appendChatText(jid.getNick() + " says:", nameColor(jid.getNick()), sResponse, COLOR_MSG);
                chatHistory(jid.getNick() + " says:", nameColor(jid.getNick()), sResponse, COLOR_MSG);
            }
        }
        if (!bGroupChat)
            lblUsername.setText(JIDYou.getNick());
    }

    /**
     * Display system msg
     */
    public void systemMsg(String sMsg) {
        txtChatHistory.append(sMsg, COLOR_STATUS);
        chatHistory(sMsg, COLOR_STATUS);
    }

    /**
     * Send indication user has started replying
     */
    public void msgReplyStarted(JID jid, String msg) {
        lblUsername.setText(JIDYou.getNick() + " " + msg);
    }

    /**
     * Change status of person we're chatting with.
     * @param sStatus New status of user we're chatting with.
     */
    public void changeStatus(JID jid, String sStatus) {
        if (jid.getNick().length() == 0)  //groupchat
        {
            txtChatHistory.append(sStatus, COLOR_STATUS);
            chatHistory(sStatus, COLOR_STATUS);
        } else {
            txtChatHistory.append(jid.getNick() + " has changed status to " + sStatus, COLOR_STATUS);
            chatHistory(jid.getNick() + " has changed status to " + sStatus, COLOR_STATUS);
        }
    }

    /**
     * Pop current window to top.  Used when corresponding roster list launches 
     * a chat that's already active (and user forgot about the active window).
     */
    protected void popTop() {
        this.toFront();
    }

    /**
     * FocusListener requires focusGained and focusLost
     * for reasons unbenownst to me requestFocus method
     * isn't working at startup, therefore when window
     * boots focus defaults to 'readonly' txtChatHistory.
     * Need to force it to txtSendMsg
     * @param fe not used
     */
    public void focusGained(FocusEvent fe) {
        if( fe.getComponent() instanceof Button ) {
            //Save the text of the button with the focus
            msButtonWithFocus = ((Button)fe.getComponent()).getLabel();
        } else {
            // don't let focus stay on txtChatHistory
            // (only object with focus listener enabled)
            txtChatHistory.transferFocus();
        }
    }

    /**
     * focus lost not used, just here to fill requirements of
     * abstract class.
     * @param fe not used
     */
    public void focusLost(FocusEvent fe) {
        msButtonWithFocus = "";
    }

    // Implement WindowListener
    /**
     * windowActivated - do nothing.
     * windowClosed - do nothing.
     * windowDeactivated - do nothing.
     * windowOpened - do nothing.
     */
    public void windowActivated(java.awt.event.WindowEvent event) {}
    public void windowClosed(java.awt.event.WindowEvent event) {}
    public void windowDeactivated(java.awt.event.WindowEvent event) {}
    public void windowOpened(java.awt.event.WindowEvent event) {}

    /**
     * windowClosing. Let the applet know and dispose of self.
     * @param event the closing event
     */
    public void windowClosing(java.awt.event.WindowEvent event) {
        app.chatWindowClosing(JIDYou);       // set window to null
        setVisible(false);
        dispose();
    }

    /**
     * Chat window state changed to deiconified
     * @param event Deiconify event
     */
    public void windowDeiconified(java.awt.event.WindowEvent event) {
        Object object = event.getSource();
        if (object == MsgChat.this) {
            MsgChat_WindowDeiconified(event);
        }
    }

    /**
     * Chat window state changed to iconified
     * 
     * @param event Window iconified event
     */
    public void windowIconified(java.awt.event.WindowEvent event) {
        Object object = event.getSource();
        if (object == MsgChat.this) {
            MsgChat_WindowIconified(event);
        }
    }

    /**
     * Send message triggered.
     * Take from the send message field and add to chat history
     * and also send it to the antenna app to be sent on to server.
     */
    private void msgSend() {
        if (txtSendMsg.getText().length() > 0) {
            String msg = txtSendMsg.getText();
            txtSendMsg.setText("");
            msgReceived(JIDMe,msg);
            app.msgSend(JIDYou, moCleanup.Cleanup(msg, false));
        }
        txtSendMsg.requestFocus();     // keep focus in text input area
        nMsgCount = 0;
        updateCount(nMsgCount);
        //mbNotifiedStartingReply = false;
    }

    /**
     * Invite button has been selected, display list of users to invite
     */
    public void inviteeSelected(String sXML) {
        MiniXMLParser oParser = new MiniXMLParser();
        String sGroupChat = oParser.parseTag(sXML, "GroupChat");
        String sReason = oParser.parseTag(sXML, "Reason");
        String sInvitee = oParser.parseTag(sXML, "Invitee");
        int indexOfColon = sInvitee.indexOf(":");
        if( indexOfColon < 0 ) {
            app.inviteUser(sGroupChat, sInvitee, sReason);
        } else {
            int i2 = 0;
            while( indexOfColon > -1 ) {
                String invitee = sInvitee.substring(i2,indexOfColon);
                app.inviteUser(sGroupChat, invitee, sReason);
                i2 = indexOfColon+1;
                indexOfColon = sInvitee.indexOf(":",i2);
            }
            // invite the last one, after the last colon.
            String invitee = sInvitee.substring(i2);
            app.inviteUser(sGroupChat, invitee, sReason);
        }
    }

    /**
     * Invite users to groupchat
     */
    private void invite() {
        InputDialog oInput = new InputDialog(this, clientColor, clientFont, "Invite to Chat");
        oInput.addHidden("GroupChat", JIDYou.getUsername());
        oInput.addQuestion("Reason for Invitation:", "Reason", "Join us in " +  JIDYou.getUsername());
        oInput.addList("Invite:", "Invitee", app.rosterListToXML(), 5,true);
        oInput.addButton("OK", "inviteeSelected");	// Call inviteeSelected when OK clicked
        oInput.addButton("Cancel", "");				// When cancelled, no need to call anything
        oInput.inputbox();
    }

    /**
     * Record/Stop button pressed
     */
    private void record(String actionCommand) {
        if( actionCommand.equals(" Record ") ) {
            mbRecord = true;
            btnRecord.setLabel("Stop Rec");
            updateCount(nMsgCount);
        } else if( actionCommand.equals("Stop Rec") ) {
            btnRecord.setLabel(" Record ");
            mbRecord = false;
            updateCount(nMsgCount);
            displayChatHistory();
        }
    }

    /**
     * Window iconified, set corresponding state variable
     * @param event window iconifiy event
     */
    private void MsgChat_WindowIconified(java.awt.event.WindowEvent event) {
        bIconified = true;
        updateCount(nMsgCount);
    }

    /**
     * Update non-seen message count
     */
    private void updateCount(int n) {
        String title = "";
        if (n == 0)
            title = JIDYou.getNick();
        else
            title = "(" + n + ") " + JIDYou.getNick();

        if( mbRecord ) {
            title += ", Recording...";
        }
        this.setTitle(title);
    }

    /**
     * Window deiconified, set window state variablle
     * @param event window deiconified event
     */
    private void MsgChat_WindowDeiconified(java.awt.event.WindowEvent event) {
        bIconified = false;
        nMsgCount = 0;
        updateCount(nMsgCount);
    }

    class SymComponent extends java.awt.event.ComponentAdapter {
        public void componentResized(java.awt.event.ComponentEvent event) {
            Object object = event.getSource();
            if (object == MsgChat.this)
                MsgChat_ComponentResized(event);
        }
    }

    /**
     * Window Resized, insure we're not too small
     * @param event Window resize event
     */
    public void MsgChat_ComponentResized(java.awt.event.ComponentEvent event) {
        if (this.getSize().width < 200) {
            this.setSize(200, this.getSize().height);
        }
        if (this.getSize().height < 200) {
            this.setSize(this.getSize().width, 200);
        }
    }

    /**
     * Add a new status line to the Chat History
     * @param sMsg			What the person said
     * @param sMsgColor		Color to use for the message
     */
    private void chatHistory(String sMsg, String sMsgColor) {
        chatHistory("", "", sMsg, sMsgColor);
    }

    /**
     * Add a new chat line to the Chat History
     * @param sName			Name of the person who said the message
     * @param sNameColor	Color to use for the name "rrggbb" format
     * @param sMsg			What the person said
     * @param sMsgColor		Color to use for the message
     */
    private void chatHistory(String sName, String sNameColor, String sMsg, String sMsgColor) {
        if (!mbRecord) {
            return;
        }

        // Check the capacity of the buffer to hold this line.
        // If it is not enough, launch the textArea to clear
        // up the buffer.
        int nameLen = 0;
        int msgLen = 0;
        if( sName != null )
            nameLen += sName.length();
        if( sMsg != null )
            msgLen += sMsg.length();
        if( (nameLen+msgLen+20+historyBuff.length()) > historyBuff.capacity() )
            displayChatHistory();

        //Don't indent status or action messages
        if ( sName.length() > 0 ) {
            historyBuff.append(sName);
            // indent up to 20 total characters.
            int pad = 20 - nameLen;
            for( int i = 0; i < pad; i++ ) {
                historyBuff.append(" ");
            }
        }
        historyBuff.append(sMsg+NEWLINE);
    }

    /**
     * Construct a properly formed HTML page with a table containing the Chat History
     * @return HTML version of Chat History
     */
    private String chatHistoryToString() {
        String s = historyBuff.toString();
        historyBuff = new StringBuffer(SB_MAX_CAPACITY);
        return s;
    }
}
