package org.jabber.applet.communication;
import org.jabber.applet.tools.*;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ 
 * 
 * or at
 * 
 * http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/


/**
* This class handles only simple Jabber Command XML parsing (meant to be small)
* 
* @author    Jonathan Paulson
* @version     $ 0.7.3 $
*/
public class MiniJabberParser {
    private MiniXMLParser   moXMLParser            = null;
    private IJabberClient   moClient                = null;
    private boolean         mbLookingForBeginTag    = true;
    private String          msXMLDoc                = "";
    private String          msTagName               = "";
    private int             miBracketNbr            = 0;
    private static String   MESSAGE                 = "message";
    private static String   PRESENCE                = "presence";
    private static String   INFO_QUERY              = "iq";
    private static String   STREAM                  = "stream:stream";

    /**
     * Class Constructor
     *
     * @param client    Reference to the Object that implements IJabberClient 
     *                    Used to communicate back to the object 
     */
    public MiniJabberParser(IJabberClient client) {
        moClient = client;
        moXMLParser    = new MiniXMLParser();
    }

    /**
    * This is a VERY simple Jabber parser.   
    * If the XML tag isn't supported then it's ignored but 
    * calls IClient.unsupportedTag() to allow the client to 
    * handle if they want to
    * 
    * @param sXML       XML to parse
    */
    public synchronized void ProcessJabber(String XML, boolean mbEchoSentXLM) {
        msXMLDoc += XML;

        // remove chars before the begin tag
        while (msXMLDoc.length()>0 & msXMLDoc.charAt(0)!='<') {
            msXMLDoc = msXMLDoc.substring(1);
        }

        // iterate over the tags to see where first tag is closed
        String toProcess = msXMLDoc;
        String firstTag = "";
        int tagsOpen = 0;
        int whereOpened = 0;
        int whereWhitespace = 0;
        while ((whereOpened = toProcess.indexOf('<')) != -1) {
            toProcess = toProcess.substring(whereOpened+1);
            int whereClosed = toProcess.indexOf('>');

            // tag not closed yet ... need more input
            if (whereClosed == -1) {
                return;
            }

            // extract the tag and check if ignored tag
            String thisTag = toProcess.substring(0, whereClosed);
            if (thisTag.charAt(0) == '/') {
                thisTag = thisTag.substring(1);
            }
            if ((whereWhitespace = thisTag.indexOf(' ')) != -1) {
                thisTag = thisTag.substring(0, whereWhitespace);
            }
            if ((whereWhitespace = thisTag.indexOf('\t')) != -1) {
                thisTag = thisTag.substring(0, whereWhitespace);
            }
            if (thisTag.toLowerCase().equals("?xml")) {
                // This comes at the start of the stream and will not be closed
                // until its end. Ignore it.
                continue;
            }

            // if it's the first tag remember it
            if (tagsOpen == 0) {
                firstTag = thisTag;
            }

            if (toProcess.charAt(0) == '/') {
                // closing tag
                tagsOpen--;
            } else if (toProcess.charAt(whereClosed-1)=='/') {
                // self closing tag
            } else {
                // opening tag
                tagsOpen++;
            }

            // is the first tag now closed?
            if ((tagsOpen == 0) || (firstTag.equals(STREAM))) {
                int sizeAfterClose = toProcess.length()-whereClosed-1;
                String thisBlock = msXMLDoc.substring(0, msXMLDoc.length() - sizeAfterClose);
                msXMLDoc = msXMLDoc.substring(msXMLDoc.length() - sizeAfterClose);
                if (firstTag.equals(MESSAGE)) {
                    processMessage(thisBlock);
                } else if (firstTag.equals(PRESENCE)) {
                    processPrescence(thisBlock);
                } else if (firstTag.equals(INFO_QUERY)) {
                    processInfoQuery(thisBlock);
                } else if (firstTag.equals(STREAM)) {
                    processStreamTag(thisBlock);
                    tagsOpen = 0; // stream:stream will not close until end of connection
                } else {
                    // Not supported
                    if (moClient != null) {
                        moClient.unsupportedTag(firstTag, thisBlock);
                    }
                }
            }
        }
    }

    /**
    * Parse out the Jabber Message elements and call
    * IClient.msgReceived()
    * 
    * @param XMLDoc       XML to parse
    */
    private void processMessage(String XMLDoc) {
        //We have the whole Message, so now parse apart the tags
        if (moClient != null) {
            String from = moXMLParser.parseTag(XMLDoc, "from");
            String sType = moXMLParser.parseTag(XMLDoc, "type");
            String sSubject = moXMLParser.parseTag(XMLDoc, "subject");
            String sBody = moXMLParser.parseTag(XMLDoc, "body");
            String sThread = moXMLParser.parseTag(XMLDoc, "thread");

            //Check to see if it's JabberIM's special X messages
            /*
            String sX = moXMLParser.parseTag(XMLDoc, "x", true);  
            if (sX.length() > 0){
                String sXMLns = moXMLParser.parseTag(sX, "xmlns");
                if (sXMLns.indexOf("x") != -1){
                    String sComposing = moXMLParser.parseTag(XMLDoc, "composing", true);
                    String sReply = moXMLParser.parseTag(sComposing, "reply", true);
                    if (sReply.length() > 0){
                        moClient.msgReplyStarted(from, sReply);
                        return;        
                    }                    
                }
            }
            */
            moClient.msgReceived(from, sType, sSubject, sBody, sThread);
        }
    }

    /**
    * Parse out the Jabber Prescence elements and call
    * IClient.presenceReceived()
    * 
    * @param XMLDoc       XML to parse
    */
    private void processPrescence(String XMLDoc) {
        //We have the whole Presence, so now parse apart the tags
        if (moClient != null) {
            String sTo = moXMLParser.parseTag(XMLDoc, "to");
            String sFrom = moXMLParser.parseTag(XMLDoc, "from");
            String sType = moXMLParser.parseTag(XMLDoc, "type");
            String sStatus = moXMLParser.parseTag(XMLDoc, "status");
            String sPriority = moXMLParser.parseTag(XMLDoc, "priority");
            String sShow = moXMLParser.parseTag(XMLDoc, "show");
            String sError = moXMLParser.parseTag(XMLDoc, "error");
            String sCode = moXMLParser.parseTag(XMLDoc, "code");

            moClient.presenceReceived(sTo, sFrom, sType, sStatus, sPriority, sShow, sError, sCode);
        }
    }

    /**
    * Parse out the Jabber InfoQuery elements and call
    * IClient.infoQueryReceived()
    * 
    * @param XMLDoc       XML to parse
    */
    private void processInfoQuery(String XMLDoc) {
        //We have the whole InfoQuery, so now parse apart the tags
        if (moClient != null) {
            String to = moXMLParser.parseTag(XMLDoc, "to");
            String sFrom = moXMLParser.parseTag(XMLDoc, "from");
            String sID = moXMLParser.parseTag(XMLDoc, "id");
            String sType = moXMLParser.parseTag(XMLDoc, "type");
            String sQuery = moXMLParser.parseTag(XMLDoc, "query");
            String sError = moXMLParser.parseTag(XMLDoc, "error");
            String sCode = moXMLParser.parseTag(XMLDoc, "code");
            String sUser = "";

            //Test to see if it is a user we need to add to the chat Hashtable
            //Check substrings in JabberApplet.java infoQueryReceived()
            if (sQuery.length() == 0) {
                sUser = moXMLParser.parseTag(XMLDoc, "conference");
                if (sUser.length() == 0) {
                    sUser = moXMLParser.parseTag(XMLDoc, "iq");
                }

                if (sUser.length() > 0) {
                    System.out.println("User: " + sUser);
                    sQuery = sUser;
                }
            }

            moClient.infoQueryReceived(to, sFrom, sID, sType, sQuery, sError, sCode);
        }
    }

    /**
    * Parse out the stream id
    * 
    * @param XMLDoc       XML to parse
    */
    private void processStreamTag(String XMLDoc) {
        if (moClient != null) {
            moClient.gotStreamID(moXMLParser.parseTag(XMLDoc, "id"));
        }
    }
}
